@@ -16,11 +16,11 @@
 """ The _PlotWindow class, used to define a plot window. """
 
 from subprocess import Popen, PIPE, DEVNULL, TimeoutExpired
-from os import path, mkdir
-from time import sleep
+from os import remove, path, mkdir
 
 from .global_variables import *
 from .errors import *
+from .funcutils import *
 
 
 # +---------+
@@ -41,7 +41,8 @@ class _PlotWindow:
                  plot_type=DEFAULT_TYPE,
                  title=None,
                  persistence=PERSISTENCE,
-                 redirect_output=REDIRECT_OUT):
+                 redirect_output=REDIRECT_OUT,
+                 purge=PURGE_DATA):
         """ An instance of this class contains a plot window.
 
             Parameters
@@ -57,6 +58,7 @@ class _PlotWindow:
             height:          height of the plot window
             xpos:            x position of the plot on the screen
             ypos:            y position of the plot on the screen
+            options:         a string containing other options for the terminal
             plot_type:       a string defining the type of plot
                              must be one of the strings in PLOT_TYPES
             title:           a string used as the title of the plot
@@ -65,7 +67,8 @@ class _PlotWindow:
             redirect_output: True:  save gnuplot otuput and errors to files
                              False: send then to /dev/stdout and /dev/stderr
                              None:  send them to /dev/null
-            options:         a string containing other options for the terminal
+            purge:           if True, old datafiles are deleted each time 
+                             new data is plotted on this window
 
             Initialized data attributes
             ---------------------------
@@ -89,6 +92,10 @@ class _PlotWindow:
             self.title:           the window title (None if not given)
             self.data_filenames:  list containing the names of the data files
             self.functions:       list containing the functions plotted
+            self.n_volatiles      number of curves plotted as volatile
+                                  (passing the data inline, without writing to file)
+            self.purge:           if True, old datafiles are deleted each time 
+                                  new data is plotted
             self.error:           one of the tuples in the errors.py module
         """
 
@@ -115,6 +122,15 @@ class _PlotWindow:
                         + DEFAULT_TYPE + '\"' )  
             self.error = (status, message)
 
+        # Store the persistence status
+        self.persistence = persistence
+
+        # Store the requested purge status
+        self.purge = purge
+        
+        # Store the window title (may be None)
+        self.title = title        
+
         # Store the number of axis     
         if (self.plot_type == '3D'): self.n_axes = 3
         else:                        self.n_axes = 2
@@ -128,19 +144,14 @@ class _PlotWindow:
                 break
         if (self.window_number is None): self.window_number = len(window_list)
         
-        # Add this plot to the list of the active plots
+        # Add this plot window to the list of the active ones
         window_list.append(self)
-
-        # Store the window title (may be None)
-        self.title = title
-
-        # Store the persistence status
-        self.persistence = persistence
         
         # Initialize the lists where data filenames 
         # and functon strings will be stored
         self.data_filenames = []
         self.functions      = []
+        self.n_volatiles    = 0
 
         # Initialize the axis ranges
         self.xmin = None
@@ -148,18 +159,15 @@ class _PlotWindow:
         self.ymin = None
         self.ymax = None
         self.zmin = None
-        self.zmax = None
-            
-        # Create the data files directory, if it doesn't exist
-        if not path.exists(DIRNAME):        mkdir(DIRNAME)
-        if not path.exists(DIRNAME_DATA):   mkdir(DIRNAME_DATA)
+        self.zmax = None            
                 
         # Start the gnuplot process to plot on this window
         if persistence: exec_list = ['gnuplot','-p']
         else:           exec_list = ['gnuplot']        
         if redirect_output is True:
             # If redirection of gnuplot output to files is requested
-            # create the directories to save gnuplot output and errors
+            # Create the directories to save gnuplot output and errors
+            if not path.exists(DIRNAME):     mkdir(DIRNAME)
             if not path.exists(DIRNAME_OUT): mkdir(DIRNAME_OUT)
             if not path.exists(DIRNAME_ERR): mkdir(DIRNAME_ERR)
             self.filename_out = path.join( DIRNAME_OUT, FILENAME_OUT
@@ -167,12 +175,8 @@ class _PlotWindow:
             self.filename_err = path.join( DIRNAME_ERR, FILENAME_ERR
                                            + '_w_' + str(self.window_number) )
             if self.title is not None:
-                self.filename_out += ( '('
-                                       + self._correct_filename(self.title)
-                                       + ')' )
-                self.filename_err += ( '('
-                                       + self._correct_filename(self.title)
-                                       + ')' )
+                self.filename_out += '(' + correct_filename(self.title) + ')'
+                self.filename_err += '(' + correct_filename(self.title) + ')'
             self.gnuplot_process = Popen(exec_list,
                                          stdin=PIPE,
                                          stdout=open(self.filename_out, 'w'),
@@ -200,8 +204,7 @@ class _PlotWindow:
                 command_string += ' ' + str(options)
             self._command(command_string)
         if title is not None:
-            self._command('set title \"' + self._correct_filename(self.title)
-                          + '\"')
+            self._command('set title \"' + correct_filename(self.title) + '\"')
             
             
     def _command(self, command_string):
@@ -235,90 +238,8 @@ class _PlotWindow:
             self.gnuplot_process.kill()
             gnuplot_last_output = self.gnuplot_process.communicate()
 
-        return gnuplot_last_output                
-
-
-    def _correct_filename(self, filename):
-        """ Remove invalid characters from a filename.
-
-            If the string contains characters in INVALID_CHARS, they are
-            substituted with the char in SUBSTITUTE_CHAR.
-
-            Parameters
-            ----------
-
-            filename: a string, representive a filename
-
-            Returns
-            -------
-
-            The modified string
-        """
-
-        for c in INVALID_CHARS:
-            filename = filename.replace(c, SUBSTITUTE_CHAR)
-
-        return filename
-    
+        return gnuplot_last_output                    
         
-    def _data_file_2d(self, x_data, y_data, filename, sep=SEP, eol=EOL):
-        """ Create a file and write 2d data to it in csv format.
-
-            The file will contain data in ascii format, in the form:
-
-            x1  y1
-            x2  y2
-            ..  ..
-
-            Parameters
-            ----------
-
-            x_data:   data representing x coordinates of the points to plot
-            y_data:   data representing y coordinates of the points to plot
-            filename: the name of the data file to be created
-            sep:      separator character between colums (vaues in a line)
-            eol:      end-of-line character
-        """
-         
-        data_file = open(filename, 'w') 
-        for i in range(len(x_data)):
-            data_file.write( str(x_data[i])
-                             + sep
-                             + str(y_data[i])
-                             + eol )
-        data_file.close()
-
-
-    def _data_file_3d(self, x_data, y_data, z_data, filename, sep=SEP, eol=EOL):
-        """ Create a file and write 3d data to it in csv format.
-
-            The file will contain data in ascii format, in the form:
-
-            x1  y1  z1
-            x2  y2  z2
-            ..  ..  ..
-
-            Parameters
-            ----------
-
-            x_data:   data representing x coordinates of the points to plot
-            y_data:   data representing y coordinates of the points to plot
-            z_data:   data representing z coordinates of the points to plot
-            filename: the name of the data file to be created
-            sep:      separator character between colums (vaues in a line)
-            eol:      end-of-line character
-        """
-         
-        data_file = open(filename, 'w') 
-        for i in range(len(x_data)):
-            data_file.write(  str(x_data[i])
-                              + sep
-                              + str(y_data[i])
-                              + sep
-                              + str(z_data[i])                                
-                              + eol )
-        data_file.close()
-
 
     def _add_functions(self, function_list, replot=False):
         """ Add one or more functions to a plot window.
@@ -347,24 +268,34 @@ class _PlotWindow:
 
         (status, message) = NOERROR
 
+        if (self.n_volatiles and replot): return ERROR_REPLOT_VOLATILE
         if not function_list: return ERROR_NO_FUNCTION 
         if (replot and (not self.data_filenames) and (not self.functions)):
-            return ERROR_NO_REPLOT           
+            return ERROR_NO_REPLOT
         # Check consistency of the whole list
         for i in range(len(function_list)):
             if ( len(function_list[i]) != 3):
                 (status, message) = ERROR_PLOT_PARAMETERS
                 if len(function_list) > 1:
                     message += ' in list item # ' + str(i)
-                message += ( ': 3 items expected, given '
-                             + str(len(function_list[i])) )
+                message += ': 3 items expected, given ' + str(len(function_list[i]))
                 return (status, message)
                    
-        # Initialize data attributes of this plot window   
+        # If it is not a replot, reset the plot window   
         if not replot:
+            # If purge status is True, remove old datafiles
+            if self.purge:
+                for filename in self.data_filenames:
+                    try:
+                        remove(filename)
+                    except FileNotFoundError:
+                        pass
+            # Clear the list of datafiles and functions
+            # and reset the number of volatiles
             self.data_filenames.clear()
             self.functions.clear()
-
+            self.n_volatiles = 0
+            
         # Beginning of the command string to send to gnuplot
         if replot:                     command_string = 'replot '
         elif (self.plot_type == '3D'): command_string = 'splot '
@@ -394,21 +325,29 @@ class _PlotWindow:
         return status, message        
         
 
-    def _add_curves(self, data_list, replot=False):
+    def _add_curves(self, data_list, volatile=VOLATILE, replot=False):
         """ Add one or more curves from data to the plot window. 
 
             Parameters
             ----------
 
             data_list:    a list in one of the following forms:
-                          2D data: [ [x1 , y1, label1, options1],     
-                                     [x2,  y2, label2, options1]     ... ]
-                          3D data: [ [x1,  y1, z1, label1, options1], 
-                                     [x2,  y2, z2, label2, options2], ... ] 
+                          2D data: [ [x1 ,  y1, label1, options1],     
+                                     [x2,   y2, label2, options2],... ]
+                          3D data: [ [x1,   y1, z1, label1, options1], 
+                                     [x2,   y2, z2, label2, options2], ... ] 
                           where:
-                          - x1 contains the x-coordinates of the points to plot
+                          - x1 contains the x-coordinates of the points to plot;
+                            for 2D plot windows only this can be set to None, 
+                            which is useful if the data are 1D, as in the case 
+                            of boxplots, or if you want gnuplot to 
+                            automatically provide x-values.
+                            2D plots with and without x values can be mixed:
+                            [ [x1,   y1, label1, options1],     
+                              [None, y2, label2, options2],... ]  
                           - y1 contains the y-coordinates of the points to plot 
                           - z1 contains the z-coordinates of the points to plot  
+                            (for 3D plot windows only)
                           - label1 is a string, that will be used to identify the plot 
                             in the legend, or None
                           - options1 is a string, containing additional options
@@ -416,6 +355,8 @@ class _PlotWindow:
                           The form must be consisted with the type of plot 
                           that was defined at the plot window creation, 
                           otherwise an error message is returned
+            volatile:     if True, the data are not written to file, but sent 
+                          to gnuplot as volatile data using the special filename '-'
             replot:       if True, the curves are plotted without erasing 
                           previously plotted ones
 
@@ -427,29 +368,48 @@ class _PlotWindow:
 
         (status, message) = NOERROR
 
+        if (self.n_volatiles and replot): return ERROR_REPLOT_VOLATILE      
         if not data_list: return ERROR_NO_DATA
         if (replot and (not self.data_filenames) and (not self.functions)):
             return ERROR_NO_REPLOT
         
         # Check consistency of the whole list
         for i in range(len(data_list)):
-            # Transform single numbers to lists
-            try:
-                len_x = len(data_list[i][0])
-            except TypeError:
-                data_list[i][0] = [ data_list[i][0] ]
-                len_x = 1
+            
+            # Check if xdata are missing in this item
+            xmissing = False
+            if (data_list[i][0] is None):
+                xmissing = True
+                len_x = None
+                # Missing x-data are not allowed in 3D plots
+                if (self.plot_type == '3D'):
+                    (status, message) = ERROR_WRONG_TYPE
+                    if (len(data_list) > 1): message += ' in list item # ' + str(i)
+                    message += ': missing x-data in a 3D plot window'
+                    return status, message
+            else:
+                # Transform single x-numbers to lists
+                try:
+                    len_x = len(data_list[i][0])
+                except TypeError:
+                    data_list[i][0] = [ data_list[i][0] ]
+                    len_x = 1
+            # Transform single y-numbers to lists        
             try:
                 len_y = len(data_list[i][1])
             except TypeError:
                 data_list[i][1] = [ data_list[i][1] ]
                 len_y = 1
-            if self.n_axes == 3:
+            # For a 3D plot, Check z values also
+            if (self.n_axes == 3):
+                # Transform single z-numbers to lists
                 try:
                     len_z = len(data_list[i][2])
                 except TypeError:
                     data_list[i][2] = [ data_list[i][2] ]
-                    len_z = 1                
+                    len_z = 1
+                    
+            # Check consistency of the data        
             if (len(data_list[i]) != self.n_axes + 2):
                 (status, message) = ERROR_PLOT_PARAMETERS
                 if len(data_list) > 1: message += ' in list item # ' + str(i)
@@ -457,39 +417,52 @@ class _PlotWindow:
                              + ' items expected, given '
                              + str(len(data_list[i])) )
                 return (status, message)
-            elif (len_x != len_y):
+            elif ( (not xmissing) and (len_x != len_y) ):
                 (status, message) = ERROR_PLOT_PARAMETERS
                 if len(data_list) > 1:  message += ' in list item # ' + str(i)
                 message += ': x and y data have different sizes'
                 return status, message
-            elif (self.n_axes == 3) and (len_z != len_x):
+            elif ( (self.n_axes == 3) and (len_z != len_y) ):
                 (status, message) = ERROR_PLOT_PARAMETERS
                 if len(data_list) > 1:  message += ' in list item # ' + str(i)
-                message += ': x, y and z data have different sizes'
+                message += ': y and z data have different sizes'
                 return status, message               
-            elif (len_x == 0):
+            elif (len_y == 0):
                 (status, message) = ERROR_PLOT_PARAMETERS
                 if len(data_list) > 1: message += ' in list item # ' + str(i)
                 message += ': zero size data given'
-                return status, message            
+                return status, message
+
+        # Create the data files directory, if it doesn't exist,
+        # unless a volatile plot has been requested
+        if not volatile:
+            if not path.exists(DIRNAME):      mkdir(DIRNAME)
+            if not path.exists(DIRNAME_DATA): mkdir(DIRNAME_DATA)
                    
-        # Initialize data attributes of this plot window   
+        # If it is not a replot, reset the plot window   
         if not replot:
+            # If purge status is True, remove old datafiles
+            if self.purge:
+                for filename in self.data_filenames:
+                    try:
+                        remove(filename)
+                    except FileNotFoundError:
+                        pass
+            # Clear the list of datafiles and functions
+            # and reset the number of volatiles
             self.data_filenames.clear()
             self.functions.clear()
+            self.n_volatiles = 0            
         
         # Beginning of the command string to send to gnuplot
         if replot:                     command_string = 'replot '
         elif (self.plot_type == '3D'): command_string = 'splot '
         else:                          command_string = 'plot '
 
-        # Number of curves before adding the new ones
+        # Save the old number of curves before adding the new ones
         n_curves = len(self.data_filenames)          
-        for i in range(len(data_list)):
-            
-            # This number is used to create a unique filename for each curve
-            curve_number = n_curves + i
-            
+        for i in range(len(data_list)):         
+                        
             # Read the curve label, if given
             if (data_list[i][self.n_axes] is None):
                 label = None
@@ -502,32 +475,47 @@ class _PlotWindow:
             else:
                 options = str(data_list[i][self.n_axes+1])
                         
-            # Define the unique filename for the data file of this curve
-            string = FILENAME_DATA 
-            string += '_w' + str(self.window_number)
-            if (self.plot_type == '3D'): string += '_3D'
-            else:                        string += '_2D'        
-            if self.title is not None:
-                string += '(' + self._correct_filename(self.title) + ')'
-            string += '_c' + str(curve_number)
-            if label is not None:
-                string += '(' + self._correct_filename(label) + ')' 
-            string += FILENAME_DATA_EXT
-            filename = path.join( DIRNAME_DATA, string )  
- 
-            # Add filename to the list
-            self.data_filenames.append(filename)
-
-            # Write data to the file
-            if self.plot_type == '3D':
-                self._data_file_3d( data_list[i][0],
-                                    data_list[i][1],
-                                    data_list[i][2],
-                                    filename )
+            # Define the unique filename for the data file
+            # or "-" for volatile data
+            if volatile:
+                filename = FILENAME_VOLATILE
             else:
-                self._data_file_2d( data_list[i][0],
-                                    data_list[i][1],
-                                    filename )
+                curve_number = n_curves + i
+                string = FILENAME_DATA 
+                string += '_w' + str(self.window_number)
+                if (self.plot_type == '3D')   : string += '_3D'
+                elif (data_list[i][0] is None): string += '_1D'
+                else:                           string += '_2D'        
+                if self.title is not None:
+                    string += '(' + correct_filename(self.title) + ')'
+                string += '_c' + str(curve_number)
+                if label is not None:
+                    string += '(' + correct_filename(label) + ')' 
+                string += FILENAME_DATA_EXT
+                filename = path.join( DIRNAME_DATA, string )
+
+            # Add filename to the list, or increase volatile data counter
+            if volatile:
+                self.n_volatiles += 1
+            else:
+                self.data_filenames.append(filename)
+
+            # Save data to data files, unless they are volatile
+            if not volatile:
+                if (self.plot_type == '3D'):
+                    data_file_3d( data_list[i][0],
+                                  data_list[i][1],
+                                  data_list[i][2],
+                                  filename )
+
+                elif (data_list[i][0] is None):
+                    data_file_1d( data_list[i][1],
+                                  filename )
+
+                else:
+                    data_file_2d( data_list[i][0],
+                                  data_list[i][1],
+                                  filename )
 
             # Add plot command to the command string
             command_string += '\"' + filename + '\"'
@@ -541,9 +529,26 @@ class _PlotWindow:
                 command_string += ' ' + options
             if (i < len(data_list) - 1):
                 command_string += ', '
+
+        # For volatile data, add the datastrings          
+        if volatile:
+            command_string += EOL
+            for data in data_list:
+                if (self.plot_type == '3D'):
+                    command_string += data2string_3d(data[0],
+                                                     data[1],
+                                                     data[2])
+
+                elif (data[0] is None):
+                    command_string += data2string_1d(data[1])
+
+                else:
+                    command_string += data2string_2d(data[0],
+                                                     data[1])
                         
         # Send command string to gnuplot   
         self._command(command_string)
 
         return status, message
-    
+
+
